#version 120
#extension GL_EXT_gpu_shader4 : enable

#define TONEMAP_ACES

//#define VOLUMETRIC_LIGHT
#define SEA_LEVEL 60.0 //[0 10 20 30 40 50 60 70 80 90 100 110 120 130 150 170 190]
#define ATMOSPHERIC_DENSITY 1.0 //[0.0 0.5 1.0 1.5 2.0 3.0 4.0 5.0]


varying vec2 texcoord;

uniform sampler2D depthtex0;
uniform sampler2D colortex1;
uniform sampler2DShadow shadow;

uniform vec4 lightCol;
uniform vec3 sunColor;
uniform vec3 nsunColor;

uniform vec3 sunVec;
uniform float sunIntensity;
uniform float skyIntensity;
uniform float skyIntensityNight;
uniform float fogAmount;
uniform float rainStrength;
uniform ivec2 eyeBrightness;
uniform float frameTimeCounter;

#include "lib/color_transforms.glsl"
#include "lib/color_dither.glsl"
#include "lib/projections.glsl"

#include "lib/sky_gradient.glsl"

#define fsign(a)  (clamp((a)*1e35,0.,1.)*2.-1.)
float triangularize(float dither)
{
    float center = dither*2.0-1.0;
    dither = center*inversesqrt(abs(center));
    dither = max(-1.0,dither); 
    return dither-fsign(center);
}
vec3 fp10Dither(vec3 color,float dither){
	const vec3 mantissaBits = vec3(6.,6.,5.);
	vec3 exponent = floor(log2(color));
	return color + dither*exp2(-mantissaBits)*exp2(exponent);
}
float interleaved_gradientNoise(){
	return fract(52.9829189*fract(0.06711056*gl_FragCoord.x + 0.00583715*gl_FragCoord.y)+52.9829189*frameTimeCounter);
}
#define SHADOW_MAP_BIAS 0.8
float calcDistort(vec2 worlpos){
	
	vec2 pos = abs(worlpos * 1.165);
	vec2 posSQ = pos*pos;
	
	float distb = pow(posSQ.x*posSQ.x*posSQ.x + posSQ.y*posSQ.y*posSQ.y, 1.0 / 6.0);
	return 1.0/((1.0 - SHADOW_MAP_BIAS) + distb * SHADOW_MAP_BIAS);
}		

float phaseg(float x, float g){
    float g2 = g * g;
    return ( g2 * -.25 + .25) / exp2( log2(-2. * (g * x) + (1. + g2)) * 1.5 );
}

vec4 getVolumetricRays(float dither,vec3 fragpos) {		
		int maxIT = 8; 
		
		//project pixel position into projected shadowmap space
		vec3 wpos = mat3(gbufferModelViewInverse) * fragpos + gbufferModelViewInverse[3].xyz;
		vec3 fragposition = mat3(shadowModelView) * wpos + shadowModelView[3].xyz;
		fragposition = diagonal3(shadowProjection) * fragposition + shadowProjection[3].xyz;


		
		//project view origin into projected shadowmap space
		vec3 start = toShadowSpaceProjected(vec3(0.));
		
		
		//rayvector into projected shadow map space
		//we can use a projected vector because its orthographic projection
		//however we still have to send it to curved shadow map space every step
		vec3 dV = (fragposition-start)/maxIT;
		vec3 dVWorld = (wpos-gbufferModelViewInverse[3].xyz)/maxIT;
		
		//apply dither
		vec3 progress = start.xyz + dV*dither;
		vec3 progressW = gbufferModelViewInverse[3].xyz+dVWorld*dither;
		vec3 vL = vec3(0.);
		
		float SdotV = dot(sunVec,normalize(fragpos))*lightCol.a;	
		float dL = length(dV);
		float mie = max(phaseg(SdotV,0.6),3.*phaseg(SdotV,0.01));
		vec3 skyCol0 = (skyIntensity*sunColor*0.03+getSkyColor(vec3(0.,1.,0.0),mat3(gbufferModelViewInverse)*sunVec,1.))*600.*sqrt(eyeBrightness.y/255.0);
		vec3 sunColor = mie*lightCol.rgb*100.*vec3(0.8,0.9,1.0);

	

		float absorbance = 1.0;
		for (int i=0;i<maxIT;i++) {
			//project into biased shadowmap space
			float distortFactor = calcDistort(progress.xy);
			vec3 pos = vec3(progress.xy*distortFactor/0.92, progress.z);
			
			float sh = 1.0;
			if (abs(pos.x) < 1.0-0.5/2048. && abs(pos.y) < 1.0-0.5/2048){
			pos = pos*vec3(0.5,0.5,0.5/3.0)+0.5;
			sh =  shadow2D( shadow, pos).x;
			}
			float density = clamp(exp2(-max(progressW.y+cameraPosition.y-SEA_LEVEL,0.0)/30.)*1.4,0.0,1.);
			absorbance *= exp2(-density*fogAmount*dL*0.3*ATMOSPHERIC_DENSITY);
			vL += (sunColor*sh+skyCol0)*density*absorbance;

			
			
			//advance the ray
			progress += dV;
			progressW += dVWorld;
		}

	return vec4(vL*dL*fogAmount,absorbance);


		
}
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////

void main() {
/* DRAWBUFFERS:1 */
	vec3 color = texture2D(colortex1,texcoord).rgb;
	
	
	float z = texture2D(depthtex0,texcoord).x;
	#ifndef VOLUMETRIC_LIGHT
	if (z < 1.0){

		vec3 fragpos = toScreenSpace(vec3(texcoord,z));
		float dist = length(fragpos);
		vec3 np3 = mat3(gbufferModelViewInverse) * (fragpos/(dist));
		vec3 skyColor = getSkyColor(np3,mat3(gbufferModelViewInverse)*sunVec,np3.y) ;

		float fogFactorAbs = exp(-dist*fogAmount*0.3*ATMOSPHERIC_DENSITY);
		float fogFactorScat = exp(-dist*fogAmount*0.8*ATMOSPHERIC_DENSITY);
		
		vec3 col0 = normalize(skyColor)*(skyIntensity*mix(vec3(1.),vec3(0.8,0.9,1.),rainStrength)+skyIntensityNight*0.01)*2.;
		vec3 fogColor = mix(col0,skyColor,clamp(dist/512.,0.,1.));
		
		color.rgb = fogColor*(1.0-fogFactorScat)*sqrt(eyeBrightness.y/255.0)*10.+color*fogFactorAbs;
	
	}
	#endif
	
	#ifdef VOLUMETRIC_LIGHT
		vec3 fragpos = toScreenSpace(vec3(texcoord,z));
		float noise=interleaved_gradientNoise();
		float fogFactorAbs = exp(-length(fragpos)*fogAmount*0.3);
		vec4 vl = getVolumetricRays(noise,fragpos);
		color *= vl.a;
		color += vl.rgb*3.0*ATMOSPHERIC_DENSITY;
	#endif
	gl_FragData[0].rgb = color;

}